dnl -*- m4 -*-
dnl----------------------------------------------------------------------
dnl This is a common set of macros to be used in the configuration of
dnl	the GDS software
dnl----------------------------------------------------------------------

dnl ---------------------------------------------------------------------
 
AC_DEFUN([AC_NDS2C_PROG_CC],
[ AC_PROG_CC
if test "${GCC}" = "yes"
then
  touch ,t.c
  ac_dmt_default_cc_includes="`${CC} -v ,t.c 2>&1 | \
        sed -n -e '/\#include/,/End of search list\./p' | \
        egrep '^ '`"
  rm ,t.c
fi
SENDS_CC_FLAVOR="generic"
case $CC in
gcc|*/gcc) SENDS_CC_FLAVOR="gnu";;
cc|*/cc)
    case "`$CC -V 2>&1`" in
    *Sun\ C\ *)
      SENDS_CC_FLAVOR="SunPRO"
      ;;
    esac
    ;;
esac
AM_CONDITIONAL([SENDS_CC_SUNPRO],[test x$SENDS_CC_FLAVOR = xSunPRO])
])
 
dnl ---------------------------------------------------------------------
 
AC_DEFUN([AC_NDS2C_PROG_CXX],
[ AC_PROG_CXX
if test "${GXX}" = "yes"
then
  touch ,t.cc
  ac_dmt_default_cxx_includes="`${CXX} -v ,t.cc 2>&1 | \
        sed -n -e '/\#include/,/End of search list\./p' | \
        egrep '^ '`"
  rm ,t.cc
fi
SENDS_CXX_FLAVOR="generic"
case $CXX in
gcc|*/gcc|g++|*/g++) SENDS_CXX_FLAVOR="gnu";;
CC|*/CC)
    case "`$CXX -V 2>&1`" in
    *Sun\ C++*)
      SENDS_CXX_FLAVOR="SunPRO"
      ;;
    esac
    ;;
esac
AC_SYS_LARGEFILE
AM_CONDITIONAL([SENDS_CXX_SUNPRO],[test x$SENDS_CXX_FLAVOR = xSunPRO])
])

dnl======================================================================
dnl NDS2_CHECK_BITW
dnl   Check bit width
dnl======================================================================

AC_DEFUN([AC_NDS2C_CHECK_BITW],
[
  dnl ===================================================================
  dnl  Test 32/64-bit architecture
  dnl ===================================================================
bit_size=
AC_ARG_ENABLE(64bit,
	      AC_HELP_STRING([--enable-64bit], [64-bit compilation]),
	      [bit_size="-m64"]
)
AC_ARG_ENABLE(32bit,
	      AC_HELP_STRING([--enable-32bit], [32-bit compilation]),
	      [bit_size="-m32"]
)
  CXXFLAGS="$CXXFLAGS $bit_size"
  CFLAGS="$CFLAGS $bit_size"
  LDFLAGS="$LDFLAGS $bit_size"
])

dnl======================================================================
dnl NDS2_CHECK_OCTAVE
dnl   Check whether octave is supported
dnl======================================================================

AC_DEFUN([AC_NDS2C_CHECK_OCTAVE],
[
octave_vid=""
AC_MSG_CHECKING([for user specified octave version])
AC_ARG_WITH(octave_version, 
	    AC_HELP_STRING([--with-octave-version@<:@@:>@],
			   [Octave-config version suffix (octave-config-vsn)]
			   ),
	    [octave_vid="$with_octave_version"]
)
if test "$octave_vid" != ""; then
   echo "$octave_vid"
   octave_vid="-$octave_vid"
else
   echo "default"
fi
have_octave="no"
AC_MSG_CHECKING([for user specified octave installation])
AC_ARG_WITH(octave,
	    AC_HELP_STRING([--with-octave@<:@=DIR@:>@],
			   [Octave installation directory (contains bin/octave-config)]
			   ),
	    [have_octave=yes],
	    [have_octave=no]
)
octave_config=`which octave-config$octave_vid 2>/dev/null`
if test $have_octave != "yes"; then
    octave_config=`which octave-config$octave_vid 2>/dev/null`
    have_octave="yes"
elif test $with_octave = "no"; then
    have_octave="no"
    octave_config="no"
elif test $with_octave = "yes" ; then
    octave_config=`which octave-config$octave_vid 2>/dev/null`
else
    octave_config="$with_octave/bin/octave-config$octave_vid"
fi

if test "x$octave_config" != "x" -a \
        "x$octave_config" != "xno" -a \
        -x "$octave_config" ; then
   octave_bin_dir=`$octave_config -p BINDIR`
   octave_version=`$octave_config -p VERSION`
   octave_cflags=`${octave_bin_dir}/mkoctfile-${octave_version} -p CFLAGS`
   octave_m_opts_count=`echo " " ${octave_cflags} ${CFLAGS} | sed 's/ /\n/g' | egrep -e '^-m[36]' | sort | uniq | wc -l`
   if test ${octave_m_opts_count} -lt 2
   then
     have_octave="yes"
   else
     octave_config="no"
     have_octave="no"
   fi
else
   octave_config="no"
   have_octave="no"
fi
case `uname` in
Darwin)
  if test x${enable_32bit} != xyes
  then
    octave_config="no"
    have_octave="no"
  fi
  ;;
esac
AC_SUBST(octave_config)
AM_CONDITIONAL([build_octave], [test "$have_octave" = "yes"])

if test x${have_octave} = xyes
then
    OCTAVE_BINDIR="`${octave_config} -p BINDIR`"
    AC_SUBST(OCTAVE_BINDIR)

    OCTAVE_VERSION="`${octave_config} -p VERSION`"
    AC_SUBST(OCTAVE_VERSION)

    MKOCTAVE="${OCTAVE_BINDIR}/mkoctfile-${OCTAVE_VERSION}"
    AC_SUBST(MKOCTAVE)

    OCTAVECFLAGS="`${MKOCTAVE} -p CFLAGS` ${bit_size}"
    AC_SUBST(OCTAVECFLAGS)

    OCTAVELDFLAGS="`${MKOCTAVE} -p LDFLAGS` ${bit_size}"
    AC_SUBST(OCTAVELDFLAGS)

    OCTAVE="${OCTAVE_BINDIR}/octave-${OCTAVE_VERSION}"
    AC_SUBST(OCTAVE)

    OCTAVE_EXT=".mex"
    AC_SUBST(OCTAVE_EXT)
fi
echo "$octave_config"
])

dnl======================================================================
dnl SENDS_CHECK_SSL
dnl   Check if OpenSSL is installed
dnl======================================================================

AC_DEFUN([AC_NDS2C_CHECK_SSL],
[
have_ssl=
PKG_CHECK_MODULES(OPENSSL, openssl, have_ssl="yes", have_ssl="no")
if test "${have_ssl}" = "yes"
then
ssllibs="$OPENSSL_LIBS"
sslcflags="$OPENSSL_CFLAGS"
fi
if test "${have_ssl}" = "no"
then
ssllibs="-lssl"
sslcflags=""
fi
AC_SUBST(ssllibs)
AC_SUBST(sslcflags)
])

dnl======================================================================
dnl SENDS_CHECK_SASL
dnl   Check if OpenSSL is installed
dnl======================================================================

AC_DEFUN([AC_NDS2C_CHECK_SASL],
[
sasllibs=
saslcflags=
have_sasl=no
enab_gss=no
AC_MSG_CHECKING([for request SASL emulation with GSSAPI])
AC_ARG_WITH(gssapi, 
	    AC_HELP_STRING([--with-gssapi@<:@=DIR@:>@],
			   [gssapi library location @<:@GSSAPI/lib@:>@]) )
case "$with_gssapi" in
"no")
  ;;
"yes")
  saslcflags="$saslcflags -DHAVE_GSSAPI -I${gssapi_inc}/include"
  sasllibs="${sasllibs} -lgssapi_krb5"
  ;;
*)
  find_gssapi_include="$find_sasl_include /sw /usr/local /usr"
  for gssapi_inc in $find_gssapi_include; do
    if test -d "$gssapi_inc/include/gssapi"
    then
      saslcflags="$saslcflags -DHAVE_GSSAPI -I${gssapi_inc}/include"
      break
    elif test -d "$gssapi_inc/include/gssglue/gssapi"
    then
      saslcflags="$saslcflags -DHAVE_GSSAPI -I${gssapi_inc}/gssglue/include"
      break
    fi
  done
  for gssapilib in gssapi_krb5 gssglue
  do
    AC_CHECK_LIB([$gssapilib],
	         [gss_init_sec_context],
                 [sasllibs="${sasllibs} -l${gssapilib}" ; break; ])
  done
  ;;
esac
AC_MSG_RESULT( [ ${with_gssapi} ] )
AC_MSG_CHECKING([for user specified sasl library])
AC_ARG_WITH(sasl, 
	    AC_HELP_STRING([--with-sasl@<:@=DIR@:>@],
			   [sasl2 library location @<:@SASL/lib@:>@]
			   ) )
AC_MSG_RESULT( [ ${with_sasl} ] )
case "$with_sasl" in
"no")
    ;;
"yes"|"")
    AC_CHECK_LIB(sasl2, sasl_client_new, [have_sasl=yes], [have_sasl=no])
    if test "${have_sasl}" != "no"
    then
        saslcflags="${saslcflags} -DHAVE_SASL=1"
        sasllibs="${sasllibs} -lsasl2"
    else
        AC_CHECK_LIB(sasl, sasl_client_new, [have_sasl=yes], [have_sasl=no])
        if test x${have_sasl} = xyes
        then
            saslcflags="${saslcflags} -DHAVE_SASL=1"
            sasllibs="${sasllibs} -lsasl"
        fi
    fi
    if test "$have_sasl" != "no"
    then
       find_sasl_include="$find_sasl_include /sw /usr/local /usr"
       for sasl_inc in $find_sasl_include; do
       	   if test -d "$sasl_inc/include/sasl"
	   then
              saslcflags="$saslcflags -I$sasl_inc/include"
	      break
	   fi
       done
    elif test x$enab_gss != xdisable
    then
	saslcflags="$gsscflg"
	sasllibs="$gsslib"
    fi
   ;;
*)
    saslcflags="-DHAVE_SASL=1 -I${with_sasl}/include"
    sasllibs="-L${with_sasl}/lib -lsasl2"
    have_sasl="${with_sasl}"
    ;;
esac
AC_SUBST(sasllibs)
AC_SUBST(saslcflags)
])

dnl======================================================================
dnl SENDS_CHECK_METAIO
dnl   Check if METAIO is installed
dnl======================================================================

AC_DEFUN([AC_NDS2C_CHECK_METAIO],
[
have_metaio=
PKG_CHECK_MODULES(METAIO, libmetaio, have_metaio="yes", have_metaio="no")
if test "${have_metaio}" = "yes"
then
metaiolibs="$METAIO_LIBS"
metaiocflags="$METAIO_CFLAGS"
fi
if test "${have_metaio}" = "no"
then
metaiolibs="-lmetaio"
metaiocflags=""
fi
AM_CONDITIONAL([require_metaio], [test "$have_metaio" = no])
AC_SUBST(metaiolibs)
AC_SUBST(metaiocflags)
])

dnl======================================================================
dnl SENDS_BUILD_TYPE
dnl   Check build type: online, offline, ...
dnl======================================================================

AC_DEFUN([AC_NDS2C_BUILD_TYPE],
[
echo $ECHO_N "checking for SENDSBUILD ... $ECHO_C"
if test "x$buildtype" = "x"
then
  buildtype="$SENDSBUILD"
fi
case "x$buildtype" in
x)
  echo "not specified default: offline $ECHO_C"
  buildtype=offline
  ;;
xonline)
  echo "$buildtype"
  ;;
xoffline)
  echo "$buildtype"
  ;;
*)
  echo "$buildtype $ECHO_C"
  echo "SENDSBUILD must be specified as online or offline - Aborting"
  exit 1
  ;;
esac

AM_CONDITIONAL([online], [test "$buildtype" = "online"])
AM_CONDITIONAL([offline], [test "$buildtype" = "offline"])
])

dnl======================================================================
dnl SENDS_CHECK_POSIX
dnl	
dnl======================================================================

AC_DEFUN([AC_NDS2C_CHECK_POSIX],
[
posixflags="-D_POSIX_C_SOURCE=200112L"
unix98flags="-D_XOPEN_SOURCE=500"
case "${host}" in
*linux*) unix98flags="-D_XOPEN_SOURCE=600" ;;
esac
])

dnl======================================================================
dnl AC_NDS2C_FINISH - Things to be done at the very end for SENDS
dnl======================================================================
 
AC_DEFUN([AC_NDS2C_FINISH],
[
  dnl ===================================================================
  dnl  Setup $architecture
  dnl ===================================================================
architecture=
case "${host}" in
*linux*) architecture="linux" ;;
*solaris*) architecture="solaris" ;;
esac
AC_SUBST(architecture)
  dnl ===================================================================
  dnl  Setup $syslibs
  dnl ===================================================================
syslibs=
case "${host}" in
*linux*) syslibs= ;;
*solaris*) syslibs="-lrt" ;;
esac
AC_SUBST(syslibs)
  dnl ===================================================================
  dnl  Setup $sysmtlibs
  dnl ===================================================================
sysmtlibs=
case "${host}" in
*linux*) sysmtlibs="-lpthread" ;;
*solaris*) sysmtlibs="-lrt -lpthread" ;;
esac
AC_SUBST(sysmtlibs)
  dnl ===================================================================
  dnl  Setup $netlib
  dnl ===================================================================
netlib=
case "${host}" in
*linux*) netlib="" ;;
*solaris*) netlib="-lnsl -lsocket" ;;
esac
AC_SUBST(netlib)
  dnl ===================================================================
  dnl  Setup $readlinelibs
  dnl ===================================================================
readlinelibs=
case "${host}" in
*linux*) readlinelibs="-lreadline -ltermcap" ;;
*solaris*) readlinelibs="-lreadline -ltermcap" ;;
esac
AC_SUBST(readlinelibs)
  dnl ===================================================================
  dnl  Specify warning message level
  dnl ===================================================================
case "$SENDS_CC_FLAVOR" in
gnu)
  CFLAGS="$CFLAGS -pipe -fPIC -Wall $posixflags $unix98flags"
  ;;
SunPRO)
  CFLAGS="$CFLAGS -v -KPIC -xc99=none"
  ;;
esac
case "$SENDS_CXX_FLAVOR" in
gnu)
  CXXFLAGS="$CXXFLAGS -pipe -fPIC -Wall -Woverloaded-virtual $posixflags $unix98flags"
  ;;
SunPRO)
  CXXFLAGS="$CXXFLAGS -KPIC"
  ;;
esac
  dnl ===================================================================
  dnl  Specify optimization
  dnl ===================================================================
opt="-O3"
hiopt="-O6"
noopt="-O0"
AC_SUBST(opt)
AC_SUBST(hiopt)
AC_SUBST(noopt)
  dnl ===================================================================
  dnl  Writing config files
  dnl ===================================================================
  dnl AC_CONFIG_FILES(config/PConfig.h:config/PConfig.in)
])
